<?php
/*  ------------------------------------------------------------------------
    awzMB Guestbook/Weblog System
    
    Homepage: http://www.awzhome.de/
    ------------------------------------------------------------------------
    
    The contents of this file are subject to the Mozilla Public License
    Version 1.1 (the "License"); you may not use this file except in
    compliance with the License. You may obtain a copy of the License at
    http://www.mozilla.org/MPL/
    
    Software distributed under the License is distributed on an "AS IS"
    basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
    License for the specific language governing rights and limitations
    under the License.

    The Initial Developer of the Original Code is Andreas Weizel.
    Portions created by the Initial Developer are
    Copyright (C) 2001-2007 Andreas Weizel. All Rights Reserved.
    
    Contributor(s):
      Andreas Weizel <webmaster@awzhome.de> (original author)
    
    Alternatively, the contents of this file may be used under the terms
    of the GNU General Public License Version 2.0 or later (the "GPL"),
    or the GNU Lesser General Public License Version 2.1 or later (the
    "LGPL"), in which case the provisions of the GPL or the LGPL are
    applicable instead of those above. If you wish to allow use of your
    version of this file only under the terms of the GPL or the LGPL and
    not to allow others to use your version of this file under the MPL,
    indicate your decision by deleting the provisions above and replace
    them with the notice and other provisions required by the GPL or the
    LGPL. If you do not delete the provisions above, a recipient may use
    your version of this file under the terms of any of the MPL, the GPL
    or the LGPL.
    ------------------------------------------------------------------------
*/

global $CurrentUser, $LangRes, $UserInputCache, $UserSeenTopics, $GlobArr, $Setting;

$CurrentUser = null;
$LangRes = null;
$UserInputCache = array();
$UserSeenTopics = array();

if (!isset($Setting['OPT_includepath'])) $Setting['OPT_includepath'] = '';

include_once($Setting['OPT_includepath'].'awzmb/modules/core/constants.incl.php');
include_once($Setting['OPT_includepath'].'awzmb/modules/core/version.incl.php');

if (isset($Setting[OPT_SETTINGSFILE]))
{
  include_once($Setting[OPT_INCLUDEPATH].'awzmb/modules/core/'.$Setting[OPT_SETTINGSFILE]);
}
else
{
  if (file_exists($Setting[OPT_INCLUDEPATH].'DEBUG'))
  {
    include_once($Setting[OPT_INCLUDEPATH].'awzmb/modules/core/settings_debug.incl.php');
  }
  else
  {
    include_once($Setting[OPT_INCLUDEPATH].'awzmb/modules/core/settings.incl.php');
  }
}

AMB_SetLangResource('core');

// Little helper for URL parameters
$GlobArr = $GLOBALS['_'.$_SERVER['REQUEST_METHOD']];

function AMB_GetXMailer() {
  return AMBVERSION.' (c) '.AMBCOPYRIGHT;
}

class AMBLangResource {
  var $langres;
  var $desc;
  
  function AMBLangResource($desc) {
    $this->langres = array();
    $this->desc = $desc;
  }
  
  function AddRes($resid, $contents) {
    $this->langres[$resid] = $contents;
  }
  
  function GetRes($resid, $substs = null) {
    return $this->InsertResParams($this->langres[$resid], $substs);
  }
  
  function InsertResParams($resstr, $substs = null) {
    $result = $resstr;
    if (($substs != null) && is_array($substs)) {
      for ($i = 1; $i <= sizeof($substs); $i++) {
        $result = str_replace('%('.((string)$i).')', $substs[$i], $result);
      }
    }
    return $result;
  }
  
  function GetDescription() {
    return $this->desc;
  }
}

class AMBUser {
  var $Name;
  var $Pwd;
  var $GBTitle;
  var $GBDescription;
  var $LogoPic;
  var $BackgroundPic;
  var $BackgroundColor;
  var $TextColor;
  var $UseCSS;
  var $CSSFile;
  var $LinkColor;
  var $GBTitleColor;
  var $EntryHead;
  var $EntryBody;
  var $UseSmilies;
  var $AllowHTML;
  var $EntriesPerPage;
  var $CommentsPerPage;
  var $CommentHeading;
  var $Censoring;
  var $MailMe;
  var $MailAddress;
  var $UseAsBlog;
  var $Flags;
  var $InputFields;
  var $ComInputFields;
  var $EssentialFields;
  var $Language;
  
  function IsFlag($flagindex) {
    return AMB_IsFlag($this->Flags, $flagindex);
  }
  
  function IsInputField($fieldindex) {
    return AMB_IsFlag($this->InputFields, $fieldindex);
  }
  
  function IsComInputField($fieldindex) {
    return AMB_IsFlag($this->ComInputFields, $fieldindex);
  }
  
  function IsInputFieldGen($fieldindex, $iscomment) {
    if ($iscomment) {
      return $this->IsComInputField($fieldindex);
    } else {
      return $this->IsInputField($fieldindex);
    }
  }
  
  function IsFieldSaved($fieldindex, $iscomment) {
    if ($iscomment) {
      return $this->IsComInputField($fieldindex);
    } else {
      return $this->IsInputField($fieldindex);
    }
  }
  
  function IsEssentialField($fieldindex) {
    return AMB_IsFlag($this->EssentialFields, $fieldindex);
  }
  
  function IsSuperAdmin() {
    global $Setting;
    return ($this->Name == $Setting[CONF_SUPERADMIN]);
  }
}

/* AMB_mysql_connect: Wrapper function for MySQL database connections
   ===================================================================
   Returns 0 if successful, 1 if no connection could be established or 2 if the database couldn't be selected.
   When error occurs and the function is not in silent mode it shows an error message and exits the script immidiately.
   On success the connection handle is stored in the global variable $MySQLCon.
*/
global $MySQLCon;
$MySQLCon = 0;
function AMB_mysql_connect($silent = false)
{
  global $LangRes, $MySQLCon, $Setting;
  $MySQLCon = @mysql_connect($Setting[CONF_SERVER], $Setting[CONF_USER], $Setting[CONF_PWD]);
  if (!$MySQLCon)
  {
    if (!$silent)
    {
      AMB_ShowHTMLHeading();
      echo('<h1 id="GBTitle">'.AMB_DataOutput($LangRes->GetRes(1001))."</h1>\n");
      $tempout = AMB_DataOutput($LangRes->GetRes(1002));
      echo('<p class="InfoLinesClass">'.$LangRes->InsertResParams($tempout, array(1=>'<a href="mailto:'.AMB_MaskMailAddress($Setting[CONF_ADMINMAIL]).'">'.AMB_MaskMailAddress($Setting[CONF_ADMINMAIL]).'</a>')).'<br /><br /></p>');
      AMB_ShowHTMLEnding();
      exit();
    }
    return 1;
  }
  else
  {
    $dbselected = @mysql_select_db($Setting[CONF_DATABASE], $MySQLCon);
    if (!$dbselected)
    {
      if (!$silent)
      {
        AMB_ShowHTMLHeading();
        echo('<h1 id="GBTitle">'.AMB_DataOutput($LangRes->GetRes(1001))."</h1>\n");
        $tempout = AMB_DataOutput($LangRes->GetRes(1003));
        echo('<p class="InfoLinesClass">'.$LangRes->InsertResParams($tempout, array(1=>'<a href="mailto:'.AMB_MaskMailAddress($Setting[CONF_ADMINMAIL]).'">'.AMB_MaskMailAddress($Setting[CONF_ADMINMAIL]).'</a>')).'<br /><br /></p>');
        AMB_ShowHTMLEnding();
        exit();
      }
      return 2;
    }
  }
  return 0;
}

function AMB_mysql_close() {
  global $MySQLCon;
  if ($MySQLCon) {
    @mysql_close($MySQLCon);
    $MySQLCon = false;
  }
}

function AMB_SetLangResource($subresid) {
  global $LangRes, $Setting;
  if ($LangRes == null) {
    $LangRes = new AMBLangResource('no language set');
  }
  $l = AMB_GetCurrentLang();
  // Include language file
  if (file_exists($Setting[OPT_INCLUDEPATH].'awzmb/modules/languages/'.$l.'/language_'.$subresid.'.incl.php')) {
    include_once($Setting[OPT_INCLUDEPATH].'awzmb/modules/languages/'.$l.'/language_'.$subresid.'.incl.php');
  }
}

function AMB_GetCurrentLang() {
  global $CurrentUser, $Setting;
  $lang = '';
  // Return user's language setting whether possible
  if ($CurrentUser != null) {
    $lang = $CurrentUser->Language;
  } else {
    $lang = '';
  }
  if ($lang == '')
    $lang = $Setting[CONF_DEFAULTLANG];
  return $lang;
}

// Maskiert einige SQL-Steuerzeichen, um SQL-Injection-Angriffe zu verhindern
function AMB_SecSQL($sqldata) {
  if (!get_magic_quotes_gpc()) {
    $ret = addslashes($sqldata);
  } else {
    $ret = $sqldata;
  }
	return $ret;
}

function AMB_UnQuote($stringdata) {
  $ret = stripslashes($stringdata);
	return $ret;
}

function AMB_MagicUnQuote($stringdata) {
  if (get_magic_quotes_gpc()) {
    return stripslashes($stringdata);
  } else {
    return $stringdata;
  }
}

/* AMB_DataOutput: Prepares any data for showing it in an HTML page.
   ==================================================================
   Return value is the for HTML output prepared data. Parameters:
      $nobrs - Converts all line breaks to <br />, if true
      $notempty - If true returns a fixed space (&nbsp;) when data is empty
      $codestyle - Sets how the output should be encoded:
                   0 - no encoding
                   1 - escapes all HTML special characters and umlauts
                   2 - escapes HTML special characters only
                   3 - escapes umlauts only
      $encspaces - If true escapes spaces to show multiple spaces correctly.
*/
function AMB_DataOutput($data, $nobrs = false, $notempty = true, $codestyle = 1, $encspaces = true) {
  if (($data == '') && $notempty)
    return '&nbsp;';
  switch ($codestyle) {
    case 1: // full HTML escape
      $result = htmlentities($data);
      break;
    case 2: // convert only special characters, no umlauts etc.
      $result = htmlspecialchars($data);
      break;
    case 3: // convert only umlauts, no special characters
      $transtable = array_intersect(get_html_translation_table(HTML_ENTITIES), get_html_translation_table(HTML_SPECIALCHARS));
      $result = strtr($data, $transtable);
    default:
      $result = $data;
      break;
  }

  if ($encspaces)
    $result = ereg_replace('  ', '&nbsp;&nbsp;', $result);
  return (!$nobrs) ? nl2br($result) : $result;
}

function AMB_DataOutputEnt($data, $nobrs = false, $notempty = true, $codestyle = 1, $encspaces = true) {
  return AMB_DataOutput(html_entity_decode($data), $nobrs, $notempty, $codestyle, $encspaces);
}

// Funktion, die das Objekt des aktuellen (eingeloggten) Benutzers zurckgibt
function AMB_ReadUserInfo($username, $silent = false) {
  global $LangRes, $CurrentUser, $MySQLCon, $Setting;
  AMB_mysql_connect($silent);
  if ($MySQLCon) {
    $sql = "SELECT * FROM gbk_users WHERE username='".AMB_SecSQL($username)."'";
    $sqlquery = mysql_query($sql, $MySQLCon);
    if ($sqlquery && (@mysql_num_rows($sqlquery) > 0)) {
      // Save user information in AMBUser object
      $row = mysql_fetch_object($sqlquery);
      $CurrentUser = new AMBUser();
      $CurrentUser->Name = $row->username;
      $CurrentUser->Pwd = $row->userpwd;
      $CurrentUser->Flags = $row->flags;
      $CurrentUser->GBTitle = $row->gbtitle;
      $CurrentUser->GBDescription = $row->gbdesc;
      $CurrentUser->LogoPic = $row->logopic;
      $CurrentUser->BackgroundPic = $row->backgroundpic;
      $CurrentUser->BackgroundColor = $row->backgroundcolor;
      $CurrentUser->TextColor = $row->textcolor;
      $CurrentUser->UseCSS = $CurrentUser->IsFlag(AMBU_USECSSFILE);
      $CurrentUser->CSSFile = $row->cssfile;
      $CurrentUser->LinkColor = $row->linkcolor;
      $CurrentUser->GBTitleColor = $row->gbtitlecolor;
      $CurrentUser->EntryHead = $row->tblcolorhead;
      $CurrentUser->EntryBody = $row->tblcolorentry;
      $CurrentUser->UseSmilies = $CurrentUser->IsFlag(AMBU_USESMILIES);
      $CurrentUser->AllowHTML = $CurrentUser->IsFlag(AMBU_ALLOWBBC);
      $CurrentUser->EntriesPerPage = $row->entriesperpage;
      $CurrentUser->CommentsPerPage = $row->commentsperpage;
      $CurrentUser->CommentHeading = $row->commenthead;
      $CurrentUser->Censoring = $CurrentUser->IsFlag(AMBU_CENSOR);
      $CurrentUser->MailMe = $CurrentUser->IsFlag(AMBU_MAILME);
      $CurrentUser->MailAddress = $row->mailaddress;
      $CurrentUser->UseAsBlog = $CurrentUser->IsFlag(AMBU_USEASBLOG);
      $CurrentUser->InputFields = $row->datafields;
      $CurrentUser->ComInputFields = $row->datafieldscom;
      $CurrentUser->EssentialFields = $row->essfields;
      $CurrentUser->Language = $row->lang;
      mysql_free_result($sqlquery);
    } else {
      if ($sqlquery) mysql_free_result($sqlquery);
      // No user found -> show error message and exit
      if (!$silent) {
        if ($Setting[CONF_NOUSERPAGE]) {
          include($Setting[CONF_NOUSERPAGE]);
        } else {
          AMB_ShowHTMLHeading();
          echo('<h1 id="GBTitle">'.AMB_DataOutput($LangRes->GetRes(1004)).'</h1><p class="InfoLinesClass">'.AMB_DataOutput($LangRes->GetRes(1005)).'</p>');
          AMB_ShowHTMLEnding();
        }
        exit();
      } else {
        return false;
      }
    }
  }
  return true;
}

/* AMB_GenDateStamp: Returns an awzMB timestamp from current time.
   ================================================================
   Returns a timestamp in the awzMB format using current server time.
   It's used in database records to store entry times.
*/
function AMB_GenDateStamp() {
  return date('YmdHi');
}

function AMB_DateStampFromUTS($utsstamp) {
  return date('YmdHi', $utsstamp);
}

function AMB_UTSFromDateStamp($datestamp) {
  if (strlen($datestamp)==12) {
    return mktime((int)substr($datestamp,8,2),
                  (int)substr($datestamp,10,2),0,
                  (int)substr($datestamp,4,2),
                  (int)substr($datestamp,6,2),
                  (int)substr($datestamp,0,4));
  } else {
    return 0;
  }
}

function AMB_DecodeDateStamp($datestamp, $codestyle = 1) {
  global $LangRes;
  if (strlen($datestamp)==12) {
    return AMB_DataOutput($LangRes->GetRes(1100, array( 1=>substr($datestamp,6,2),
                                                        2=>$LangRes->GetRes(1109 + sprintf('%d', substr($datestamp,4,2))),
                                                        3=>substr($datestamp,0,4),
                                                        4=>substr($datestamp,8,2),
                                                        5=>substr($datestamp,10,2)
                                                      )
                                          ), true, true, $codestyle);
  } else {
    return '############';
  }
}

function AMB_FormatFilter($entrycode, $noformats = false, $genlinks = true) {
  global $CurrentUser, $Setting;
  // Falls gewnscht, Links automatisch unterlegen
  //$newcode = ($genlinks) ? AMB_GenLinks($entrycode) : $entrycode;
  $newcode = $entrycode;
  if ($CurrentUser->AllowHTML) {
    // Einfache Tags [b], [i], [u], [code] mit Schlietags
    $newcode = preg_replace("/\[(\/)?(b|i|u|code)\]/", ((!$noformats) ? "<$1$2>" : ""), $newcode);
    // [color=] mit Schlietag
    $newcode = preg_replace("/\[color=([\w#]+)\](.*?)\[\/color\]/s", ((!$noformats) ? "<span style=\"color: $1;\">$2</span>" : "$2"), $newcode);
    // [size=] mit Schlietag
    $newcode = preg_replace("/\[size=([\d]+)\](.*?)\[\/size\]/s", ((!$noformats) ? "<span style=\"font-size: $1pt;\">$2</span>" : "$2"), $newcode);
    // [list] mit Schlietag
    $newcode = preg_replace("/\[list\](?:<br \/>)*(.*?)\[\/list\](?:<br \/>)*/s", ((!$noformats) ? "<ul>$1</ul>" : "$2"), $newcode);
    // [list=] mit Schlietag
    $newcode = preg_replace("/\[list=(i|I|1|a|A)\](?:<br \/>)*(.*?)\[\/list\](?:<br \/>)*/s", ((!$noformats) ? "<ol start=\"1\" type=\"$1\">$2</ol>" : "$2"), $newcode);
    // [*] fr Listeneintrge
    $newcode = preg_replace("/\[\*\](.*?)$/m", ((!$noformats) ? "<li>$1</li>" : "$1"), $newcode);
    // [url] mit Schlietag
    $newcode = preg_replace("/\[url\](.*?)\[\/url\]/", ((!$noformats) ? "<a href=\"$1\"".(!$Setting[OPT_NONEWWIN] ? " target=\"_blank\"" : '').">$1</a>" : "$1"), $newcode);
    // [url=] mit Schlietag
    $newcode = preg_replace("/\[url=([^\]]+)\](.*?)\[\/url\]/s", ((!$noformats) ? "<a href=\"$1\"".(!$Setting[OPT_NONEWWIN] ? " target=\"_blank\"" : '').">$2</a>" : "$1"), $newcode);
    // [email] mit Schlietag
    $newcode = preg_replace("/\[email\](.*?)\[\/email\]/", ((!$noformats) ? "<a href=\"mailto:$1\">$1</a>" : "$1"), $newcode);
    // [img] mit Schlietag
    $newcode = preg_replace("/\[img\](.*?)\[\/img\]/", ((!$noformats) ? "<img src=\"$1\" />" : ""), $newcode);
  }
  return $newcode;
}

function AMB_InsertSmilies($text) {
  global $CurrentUser, $Setting;
  $newtext = $text;
  if ($CurrentUser->UseSmilies) {
    // icon for ";-)" and ";)"
    $newtext = eregi_replace(";\-?\)+", "<img src=\"".$Setting[CONF_PICTUREPATH]."smile2.gif\" />", $newtext);
    // icon for ":-)" and ":)"
    $newtext = eregi_replace(":\-?\)+", "<img src=\"".$Setting[CONF_PICTUREPATH]."smile.gif\" />", $newtext);
    // icon for ":-(" and ":("
    $newtext = eregi_replace("(:|;)\-?\(+", "<img src=\"".$Setting[CONF_PICTUREPATH]."sad.gif\" />", $newtext);
    // icon for ":-|" and ":|"
    $newtext = eregi_replace("(:|;)\-?\|", "<img src=\"".$Setting[CONF_PICTUREPATH]."neutral.gif\" />", $newtext);
    // icon for "*g*" & Co.
    $newtext = eregi_replace("\*(f|e|v|b)*(g)+\*", "<img src=\"".$Setting[CONF_PICTUREPATH]."grin.gif\" />", $newtext);
    // icon for "*lol*" & Co.
    $newtext = eregi_replace("(\*lol\*)|(:\-?D)", "<img src=\"".$Setting[CONF_PICTUREPATH]."lol.gif\" />", $newtext);
    // icon for "*rotfl*" & Co.
    $newtext = eregi_replace("\*rotfl\*", "<img src=\"".$Setting[CONF_PICTUREPATH]."rotfl.gif\" />", $newtext);
    // icon for "8-)" & "8)"
    $newtext = eregi_replace("8\-?\)", "<img src=\"".$Setting[CONF_PICTUREPATH]."cool.gif\" />", $newtext);
    // icon for ":P", ":-P" & Co.
    $newtext = eregi_replace(":\-?p", "<img src=\"".$Setting[CONF_PICTUREPATH]."tong.gif\" />", $newtext);
    // icon for ":?", ":-?" & Co.
    $newtext = eregi_replace(":\-?(\?|\\\\)", "<img src=\"".$Setting[CONF_PICTUREPATH]."confused.gif\" />", $newtext);
    // icon for ":x"
    $newtext = eregi_replace(":x", "<img src=\"".$Setting[CONF_PICTUREPATH]."tong.gif\" />", $newtext);
    // icon for "*oops*"
    $newtext = eregi_replace("\*oops\*", "<img src=\"".$Setting[CONF_PICTUREPATH]."oops.gif\" />", $newtext);
  }
  return $newtext;
}

/* function AMB_GenLinks($entrycode) {
  return eregi_replace("([[:alnum:]]+)://([^[:space:]'\"]*)([[:alnum:]#?/&=])",
    "<a href=\"\\1://\\2\\3\" target=\"_blank\">\\1://\\2\\3</a>", $entrycode);
} */

function AMB_GenLinks($entrycode) {
  return eregi_replace("([[:alnum:]]+)://([^[:space:]'\"]*)([[:alnum:]#?/&=])",
    "[url]\\1://\\2\\3[/url]", $entrycode);
}

function AMB_CorrectEntities($entrycode) {
  return ereg_replace("&amp;([[:alnum:]]+);","&\\1;",$entrycode);
}

/* AMB_ShowHTMLHeading: Creates the HTML head of a page using given template.
   ===========================================================================
   This function will use the page beginning template file which has to be set by the container script.
   The file name is saved in $Setting[OPT_HTMLBEGIN].
*/
function AMB_ShowHTMLHeading() {
  global $Setting, $LangRes;
  if ($Setting[OPT_HTMLBEGIN] == '') {
    if (function_exists('AMBTEMPLATEBEGIN')) {
      // For compatibility with older versions (>= 4.1)
      AMBTEMPLATEBEGIN();
    } elseif (function_exists('WCGTEMPLATEBEGIN')) {
      // For compatibility with older versions (< 4.1)
      WCGTEMPLATEBEGIN();
    } else {
      $Setting[OPT_HTMLBEGIN] = $Setting[OPT_INCLUDEPATH].'commonstyle/blankbook/gbbegin.incl.php';
    }
  }
  include_once($Setting[OPT_HTMLBEGIN]);
  echo('<div id="AMBOutput">');
  if ((AMB_GetMyScript() != 'installer.php') && file_exists($Setting[OPT_INCLUDEPATH].'installer.php')) {
    // Show warning as installer is still in awzMB's directory!
    echo('<p style="border: 2px solid #ff0000; color: #ff0000; font-weight: bold">'.AMB_DataOutput($LangRes->GetRes(1007)).'</p>');
  }
}

/* AMB_ShowHTMLEnding: Creates the HTML ending of a page using given template.
   ===========================================================================
   This function will use the page ending template file which has to be set by the container script.
   The file name is saved in $Setting[OPT_HTMLEND].
*/
function AMB_ShowHTMLEnding() {
  global $Setting;
  if ($Setting[OPT_HTMLEND] == '') {
    if (function_exists('AMBTEMPLATEEND')) {
      // For compatibility with older versions (>= 4.1)
      AMBTEMPLATEEND();
    } elseif (function_exists('WCGTEMPLATEEND')) {
      // For compatibility with older versions (< 4.1)
      WCGTEMPLATEEND();
    } else {
      $Setting[OPT_HTMLEND] = $Setting[OPT_INCLUDEPATH].'commonstyle/blankbook/gbend.incl.php';
    }
  }
	echo('<br /><hr size="1" /><table style="border-style: none"><tr><td><a href="http://www.awzhome.de" target="_blank"><img alt="AWZhome Project" style="border-style: none" src="'.$Setting[CONF_PICTUREPATH].'awzhomeproject.gif" /></a>&nbsp;</td><td style="font-size: smaller; font-style: italic"><a href="http://www.awzhome.de/?projects/awzmb" target="_blank">'.AMBVERSION.'<br />Copyright &copy; '.AMBCOPYRIGHT.'</a></td></tr></table></div>');
  include_once($Setting[OPT_HTMLEND]);
}

function AMB_GetMyScript() {
  $arr = explode('/', $_SERVER['SCRIPT_NAME']);
  return $arr[sizeof($arr) - 1];
}

function AMB_GetMyScriptAbs($noindexpage = false) {
  $scriptpath = substr($_SERVER['SCRIPT_NAME'], 1);
  $scriptname = AMB_GetMyScript();
  if ($noindexpage && ($scriptname == 'index.php')) $scriptpath = substr($scriptpath, 0, -strlen($scriptname));
  return 'http://'.$_SERVER['SERVER_NAME'].'/'.$scriptpath;
}

/* AMB_CreateLink: Creates the HTML code for a link in a awzMB page.
   ==================================================================
   This function creates a link to a awzMB page automatically including
   session IDs and $Setting[OPT_URLPARAMS] for additional parameters.
   
   Function parameters:
   
   $LinkText: Text of the link. The function encodes the text automatically.
   $LinkScript: Name of the PHP script file to be used for the link.
                If left blank the name of the current executing script is used.
   $ScriptParams: URL parameters for the script. You don't have to encode
                  special characters (like & -> &amp;) as it's done by the
                  function automatically.
   $UseSID: If true, the session ID is included in the link URL.
   
   Returns the HTML code as a string.
*/
function AMB_CreateLink($LinkText, $LinkScript, $ScriptParams, $UseSID = false) {
  $sidstring = (($UseSID) ? @strip_tags(SID) : '');
  $hrefparams = '';
  if ($sidstring != '') {
    $hrefparams = $sidstring;
  }
  $hrefparams .= AMB_CreateCustomParamCode();
  if ($ScriptParams != '') {
    $hrefparams .= (($hrefparams != '') ? '&' : '').$ScriptParams;
  }
  $hrefscript = ($LinkScript == '' ? AMB_GetMyScriptAbs(true) : $LinkScript);
  return '<a href="'.AMB_DataOutput($hrefscript.($hrefparams == '' ? '' : '?'.$hrefparams), false, false).'">'.AMB_DataOutput($LinkText).'</a>';
}

function AMB_FixWebURL($inputurl) {
  if ($inputurl == '') {
    return '';
  } else {
    if (substr($inputurl, 0, 7) != 'http://') {
      $ret = 'http://'.$inputurl;
    } else {
      $ret = $inputurl;
    }
    return AMB_DataOutput(str_replace(' ', '%20', $ret), false, false, 1, false);
  }
}

function AMB_ReadInputCache() {
  global $UserInputCache, $CurrentUser;
  if (!$CurrentUser->IsFlag(AMBU_NOINPUTCACHE) && isset($_COOKIE['awzMBCache'])) {
    $UserInputCache = unserialize(AMB_MagicUnQuote($_COOKIE['awzMBCache']));
  }
  if (!is_array($UserInputCache)) {
    $UserInputCache = array();
  } else {
    // Due to serialization there are too many backslashes in the values -> correct it
    foreach ($UserInputCache as $key => $value) {
      $UserInputCache[$key] = stripslashes($UserInputCache[$key]);
    }
  }
}

function AMB_SaveInputCache() {
  global $CurrentUser, $UserInputCache, $Setting;
  if (!$CurrentUser->IsFlag(AMBU_NOINPUTCACHE) && is_array($UserInputCache)) {
    $cookieval = @serialize($UserInputCache);
    setcookie('awzMBCache', $cookieval, time()+60*60*24*365, $Setting[CONF_COOKIEPATH], $Setting[CONF_COOKIEDOMAIN], 0);
  }
}

function AMB_ReadSeenTopics() {
  global $UserSeenTopics, $CurrentUser;
  if (isset($_COOKIE['awzMBTempCache'])) {
    $UserSeenTopics = unserialize(AMB_MagicUnQuote($_COOKIE['awzMBTempCache']));
  }
  if (!is_array($UserSeenTopics)) {
    $UserSeenTopics = array();
  } else {
    // Due to serialization there are too many backslashes in the values -> correct it
    foreach ($UserSeenTopics as $key => $value) {
      if (is_array($value)) {
        foreach ($value as $key2) {
          $UserSeenTopics[$key][$key2] = stripslashes($UserSeenTopics[$key][$key2]);
        }
      } else {
        $UserSeenTopics[$key] = stripslashes($UserSeenTopics[$key]);
      }
    }
  }
}

function AMB_IsTopicNew($entryid, $newestdate) {
  global $CurrentUser, $UserSeenTopics;
  $dstamp = AMB_GenDateStamp();
  return ($newestdate > (AMB_DateStampFromUTS(time() - 3 * 86400)));
}

function AMB_MaskMailAddress($origaddr) {
  $masked = strtolower($origaddr);
  $b = 'a';
  $trans = get_html_translation_table(HTML_ENTITIES);
  if (isset($trans['&'])) unset($trans['&']);
  $masked = str_replace('&', '&amp;', $masked);
  for ($i = 0; $i < 26; $i++) {
    $masked = str_replace($b, '&#'.(97 + $i).';', $masked);
    $b = chr(ord($b) + 1);
  }
  $masked = str_replace('.', '&#46;', $masked);
  $masked = str_replace('-', '&#45;', $masked);
  $masked = str_replace('_', '&#95;', $masked);
  $masked = str_replace('@', '&#64;', $masked);
  return strtr($masked, $trans);
}

function AMB_FixMailLinefeeds($mailbody) {
  return preg_replace("/\\r\\n/", "\n", $mailbody);
}

function AMB_IsFlag($expr, $fieldindex) {
  $fieldpow = 1 << $fieldindex;
  return (($expr & $fieldpow) == $fieldpow);
}

function AMB_GetEntrySQLFragment($username, $querycom, $censor, $noforumsort = false, $nocomcensor = false) {
  global $CurrentUser;
  $table = 'gbk';
  if ($censor) {
    $censorcond = ($nocomcensor) ? ' AND gbk.censor=1' : ' AND (gbk.censor=1 OR gbk_comments.censor=1)';
  } else {
    $censorcond = ' AND gbk.censor=0';
    //$censorcond = ($nocomcensor) ? ' AND gbk.censor=0' : ' AND gbk.censor=0 AND (gbk_comments.censor=0 OR gbk_comments.censor IS NULL)';
  }
  //if ($CurrentUser->IsFlag(14) && !$noforumsort) {
    return "(SELECT SQL_CALC_FOUND_ROWS gbk.entryid,gbk.username,gbk.name,gbk.email,gbk.hp,gbk.datestamp AS datestamp,gbk.contents,gbk.wmastcomment,gbk.censor,gbk.topic,gbk.icqnum,gbk.msn,gbk.yahoo,gbk.aim,gbk.skype,gbk.flags,COUNT(*) AS comcount,MAX(gbk_comments.datestamp) AS newest
            FROM gbk RIGHT OUTER JOIN gbk_comments USING(entryid)
            WHERE gbk.username='".AMB_SecSQL($CurrentUser->Name)."'$censorcond
            GROUP BY gbk.entryid)
            UNION ALL
            (SELECT gbk.entryid,gbk.username,gbk.name,gbk.email,gbk.hp,gbk.datestamp AS datestamp,gbk.contents,gbk.wmastcomment,gbk.censor,gbk.topic,gbk.icqnum,gbk.msn,gbk.yahoo,gbk.aim,gbk.skype,gbk.flags,COUNT(gbk_comments.comid) AS comcount,gbk.datestamp AS newest
            FROM gbk LEFT OUTER JOIN gbk_comments USING(entryid)
            WHERE gbk.username='".AMB_SecSQL($CurrentUser->Name)."' AND (gbk_comments.comid IS NULL)$censorcond
            GROUP BY gbk.entryid) ".
            (($CurrentUser->IsFlag(14) && !$noforumsort) ? 'ORDER BY flags & 2 DESC, newest DESC' : 'ORDER BY flags & 2 DESC, datestamp DESC');
  //          ORDER BY newest DESC";
  /*} else {
    return "SELECT SQL_CALC_FOUND_ROWS gbk.entryid,gbk.username,gbk.name,gbk.email,gbk.hp,gbk.datestamp,gbk.contents,gbk.wmastcomment,gbk.censor,gbk.topic,gbk.icqnum,gbk.flags,COUNT(gbk_comments.comid) AS comcount,MAX(gbk_comments.datestamp) AS newest
            FROM gbk LEFT OUTER JOIN gbk_comments USING(entryid)
            WHERE gbk.username='".AMB_SecSQL($CurrentUser->Name)."'$censorcond
            GROUP BY gbk.entryid
            ORDER BY gbk.datestamp DESC";
    
  } */
}

function AMB_CreateCSSRef($cssurl) {
  echo('<link rel="stylesheet" href="'.AMB_DataOutput($cssurl, true, false)."\" type=\"text/css\" />\n");
}

function AMB_GetAdminHelpLink($hid, $anchor) {
  global $Setting, $LangRes;
  return '&nbsp;(<a href="'.$Setting[CONF_ADMINHELPSCRIPT].'?'.$hid.'#'.$anchor.'" target="_blank" onclick="javascript:window.open(\''.$Setting[CONF_ADMINHELPSCRIPT].'?'.$hid.'#'.$anchor.'\', \'helpwindow\', \'width=400,height=200,resizable=yes,scrollbars=yes\'); return false;" title="'.$LangRes->GetRes(1602).'">?</a>)';
}

function AMB_InitColorPicker() {
  global $Setting;
  echo('<script src="'.$Setting[CONF_JSPATH].'picker.js" type="text/javascript"></script>');
  echo("<script type=\"text/javascript\"><!--\npickerPath = \"".$Setting[CONF_PICTUREPATH]."\"; altText = \"Farbe ausw&auml;hlen\";//--></script>");
}

function AMB_GetPickerHandlers() {
  return 'onchange="UpdatePicker(this.id);" onkeyup="UpdatePicker(this.id)"';
}

function AMB_CreateColorPicker($fieldid) {
  return "<script type=\"text/javascript\"><!--\nCreatePicker(\"$fieldid\"); //--></script>";
}

function AMB_CreateCustomParamCode($isform = false) {
  global $Setting;
  $result = '';
  if (is_array($Setting[OPT_URLPARAMS])) {
    if ($isform) {
      // Create hidden form fields for URL parameters
      foreach($Setting[OPT_URLPARAMS] as $par => $val) {
        $result .= "<input type=\"hidden\" name=\"".AMB_DataOutput($par, true)."\" value=\"".AMB_DataOutput($val, true)."\" />\n";
      }
    } else {
      // Create HTTP GET parameters
      foreach($Setting[OPT_URLPARAMS] as $par => $val) {
        if ($result != '')
          $result .= '&';
        $result .= $par.'='.$val;
      }
    }
  }
  return $result;
}

/* AMB_RegisterUser: Creates a new user account
   =============================================
   This function takes user name ($name), user password ($pwd), the password repetition ($pwdrep) and
   user's mail address to register the new account by adding the corresponding record to awzMB's database tables.
   The return value is an error code:
     0 - User registration was successful.
     1 - Given user name is already registered.
     2 - Given passwords don't match.
     3 - Database error occured.
     4 - Multiple registration from same client.
*/
function AMB_RegisterUser($name, $pwd, $pwdrep, $email) {
  global $LangRes, $MySQLCon;
  if (AMB_ReadUserInfo($name, true)) {
    // User already exists
    return 1;
  }
  if ($pwd != $pwdrep) {
    // Passwords don't match
    return 2;
  }
  // IP and timestamp
  $currentdate = date('YmdHis');
  $regIP = $_SERVER['REMOTE_ADDR'];
  // Check if there has been a registration with the same IP in the defined time period
  $sql = "SELECT userid FROM gbk_users WHERE regip='".AMB_SecSQL($regIP)."' AND (regdt BETWEEN DATE_SUB($currentdate, INTERVAL 1 DAY) AND $currentdate)";
  $sqlquery = mysql_query($sql, $MySQLCon);
  $identicIPs = mysql_num_rows($sqlquery);
  mysql_free_result($sqlquery);
  if ($identicIPs == 0) {
    $sql = "INSERT INTO gbk_users (username,userpwd,backgroundpic,backgroundcolor,linkcolor,textcolor,gbtitle,cssfile,gbtitlecolor,tblcolorhead,tblcolorentry,entriesperpage,mailaddress,logopic,commenthead,flags,datafields,regip,regdt) VALUES ";
    $sql .= "('".AMB_SecSQL($name)."',";
    $sql .= "'".AMB_SecSQL(md5($pwd))."',";
    $sql .= "'',";
    $sql .= "'#FFFFFF',";
    $sql .= "'#0000FF',";
    $sql .= "'#000000',";
    $sql .= "'".AMB_SecSQL($LangRes->GetRes(1600))."',";
    $sql .= "'',";
    $sql .= "'#000000',";
    $sql .= "'#D0E0F9',";
    $sql .= "'#EFF3F7',";
    $sql .= "'10',";
    $sql .= "'".AMB_SecSQL($email)."',";
    $sql .= "'',";
    $sql .= "'".AMB_SecSQL($LangRes->GetRes(1601))."',";
    $sql .= "64, 11,";
    $sql .= "'".AMB_SecSQL($regIP)."', $currentdate)";
    $result = mysql_query($sql, $MySQLCon);
    if ($result) {
      // Registration was successful
      return 0;
    } else {
      // We have a database error
      return 3;
    }
  } else {
    // IP already used in the last 24 hours
    return 4;
  }
}

/* AMB_CorrectUserName: Returns a correct user name for a created account
   =======================================================================
   Removes all non-alphanumeric characters from the string in order to get a proper user name.
*/
function AMB_CorrectUserName($name) {
  return eregi_replace("[^A-Z0-9\-]", '', $name);
}

/* AMB_DeleteUser: Deletes an user account.
   =========================================
   Takes the name of an account and deletes it from database. Additionally this function also
   removes all entries of that user still stored in the tables.
   The function returns a combination of following bit values which represent error codes:
     
     0 - Operation was executed successfully
     1 - User's entries couldn't be deleted.
     2 - User's comments couldn't be deleted.
     4 - The account couldn't be deleted.
*/
function AMB_DeleteUser($name) {
  global $LangRes, $MySQLCon;
  $result = 0;
  // Try to remove all entries for this account from the database
  $sql2 = "DELETE FROM gbk WHERE username='".AMB_SecSQL($name)."'";
  $sqlquery2 = mysql_query($sql2, $MySQLCon);

  if (!$sqlquery2 || (@mysql_affected_rows($MySQLCon) < 1)) {
    // Deleting entries failed
    $result = $result | 1;
    $entriesrem = false;
  } else {
    $entriesrem = true;
  }
  // Try to remove all comments for this account from the database
  $sql3 = "DELETE FROM gbk_comments WHERE username='".AMB_SecSQL($name)."'";
  $sqlquery3 = mysql_query($sql3, $MySQLCon);
  if (!$sqlquery3 || (@mysql_affected_rows($MySQLCon) < 1)) {
    // Deleting comments failed
    $result = $result | 2;
    $entriesrem = false;
  } else {
    $entriesrem = $entriesrem && true;
  }
  // Try to delete the account itself
  $sql = "DELETE FROM gbk_users WHERE username='".AMB_SecSQL($name)."'";
  $sqlquery = mysql_query($sql, $MySQLCon);
  
  if ($sqlquery && (@mysql_affected_rows($MySQLCon) > 0)) {
    // No error code addition, deleting account was successful
  } else {
    // Couldn't delete it...
    $result = $result | 4;
  }
  
  return $result;
}

?>